---
layout: default
title: Radically focused, CoffeeScript-oriented interface for building web apps on Node.js with Express and Socket.IO.
---

# Node development for the lazy

Zappa is a radically focused, [CoffeeScript](http://coffeescript.org)-oriented interface for building web apps on [Node.js](http://nodejs.org) with [Express](http://expressjs.com) and [Socket.IO](http://socket.io).

It turns this:

{% highlight coffeescript %}
app = require('express').createServer()
io = require('socket.io').listen(app)

app.use express.bodyParser()
app.use express.methodOverride()
app.use app.router
app.use express.static __dirname + '/public'

app.configure 'development', ->
  app.use express.errorHandler dumpExceptions: on

app.configure 'production', ->
  app.use express.errorHandler()

app.get '/:foo', (req, res) ->
  req.params.foo += 'bar'
  res.render 'index', foo: req.params.foo
  
io.sockets.on 'connection', (socket) ->
  socket.emit 'welcome', time: new Date()
  
  socket.on 'shout', (data) ->
    socket.broadcast.emit 'shout',
      id: socket.id, text: data.text

app.listen 3000
{% endhighlight %}

Into this:

{% highlight coffeescript %}
require('zappa') ->
  use 'bodyParser', 'methodOverride', app.router, 'static'

  configure
    development: -> use errorHandler: {dumpExceptions: on}
    production: -> use 'errorHandler'

  get '/:foo': ->
    @foo += 'bar'
    render 'index'
  
  at connection: ->
    emit 'welcome', time: new Date()
    
  at shout: ->
    broadcast 'shout', {id, @text}
{% endhighlight %}

## Learn More

- Get the gist with the [crash course](/docs/crashcourse)

- Check the [API reference](/docs/reference)

- See the [examples](https://github.com/mauricemach/zappa/tree/master/examples) included with the source

- Read the [annotated source](/docs/zappa.html) generated by [docco](http://jashkenas.github.com/docco/)

## Other resources

- The source code [repository](http://github.com/mauricemach/zappa) at github

- Questions, suggestions? Drop us a line on the [mailing list](http://groups.google.com/group/zappajs)

- Rather do it realtime? Join the IRC channel on freenode: [#zappajs]((irc://irc.freenode.net/zappajs)

- Found a bug? Open an [issue](http://github.com/mauricemach/zappa/issues) at github

- Check the project's history at the [change log](https://github.com/mauricemach/zappa/blob/master/CHANGELOG.md)

- Migrating from 0.1.x? See what changed in [v0.2.x](/docs/peaches), and follow the [quick guide](/docs/migration)

- Deploying to heroku? Check [this blog post](http://blog.superbigtree.com/blog/2011/08/19/hosting-zappa-0-2-x-on-heroku/)